@@ -3,18 +3,21 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.2 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.  
- * Please see the License for the specific language governing rights and 
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
  * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
@@ -38,6 +41,7 @@ private:
     task_t				fTask;
     const IOExternalMethod *		fMethods;
     IOCommandGate *			fGate;
+    IOMemoryMap *			fMemMap;
     UInt32				fNumMethods;
     mach_port_t 			fWakePort;
     bool				fDead;
@@ -75,6 +79,8 @@ public:
     virtual IOReturn			GetDebuggingLevel(KernelDebugLevel * inLevel);
     virtual IOReturn			GetDebuggingType(KernelDebuggingOutputType * inType);
     virtual IOReturn			SetTestMode(UInt32 mode, UInt32 port);
+    virtual IOReturn			ReadRegister(UInt32 offset, UInt32 size, void *value);
+    virtual IOReturn			WriteRegister(UInt32 offset, UInt32 size, UInt32 value);
 };
 
 
